<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 3/21/16
 * Time: 9:29 AM
 */

namespace qfq;
//use qfq;

require_once(__DIR__ . '/../../qfq/BodytextParser.php');
require_once(__DIR__ . '/../../qfq/exceptions/UserFormException.php');

use PHPUnit\Framework\TestCase;

//class BodytextParserTest extends \PHPUnit_Framework_TestCase {

/**
 * Class BodytextParserTest
 * @package qfq
 */
class BodytextParserTest extends TestCase {

    public function testProcessPlain() {
        $btp = new BodytextParser();

        // Simple row, nothing to remove
        $given = "10.sql = SELECT 'Hello World'";
        $expected = $given;
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Several rows, remove all but one
        $given = "\n#some comments\n10.sql = SELECT 'Hello World'\n\n    \n   #more comment";
        $expected = "10.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Several rows, all to remove
        $given = "\n#some comments\n\n\n    \n   #more comment";
        $expected = "";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Join a line
        $given = "\n10.sql = SELECT 'Hello World',\n'more content'\n      WHERE help=1";
        $expected = "10.sql = SELECT 'Hello World', 'more content' WHERE help=1";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Join several lines, incl. form
        $given = "\n10.sql = SELECT 'Hello World',\n'more content'\n      WHERE help=1\n 20.head = <table>\n 30.sql     =   SELECT\n col1,\n col2, \n col3\n  # Query stops here\nform = Person\n";
        $expected = "10.sql = SELECT 'Hello World', 'more content' WHERE help=1\n20.head = <table>\n30.sql     =   SELECT col1, col2, col3\nform = Person";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: one.
        $given = "10{\nsql = SELECT 'Hello World'\n}\n";
        $expected = "10.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: one. No LF at the end
        $given = "10{\nsql = SELECT 'Hello World'\n}";
        $expected = "10.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression, one, added some white spaces
        $given = "\n\n10 { \n \n   sql    =    SELECT 'Hello World'     \n\n    }\n\n";
        $expected = "10.sql    =    SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: multiple, simple
        $given = "10.sql = SELECT 'Hello World'\n20 {\nsql='Hello world2'\n}\n30 {\nsql='Hello world3'\n}\n";
        $expected = "10.sql = SELECT 'Hello World'\n20.sql='Hello world2'\n30.sql='Hello world3'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: complex
        $given = "10.sql = SELECT 'Hello World'\n20 {\nsql='Hello world2'\n30 { \n sql=SELECT 'Hello World3'\n40 { \n sql = SELECT 'Hello World4'\n  }  \n  } \n  }  ";
        $expected = "10.sql = SELECT 'Hello World'\n20.sql='Hello world2'\n20.30.sql=SELECT 'Hello World3'\n20.30.40.sql = SELECT 'Hello World4'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // form=...., {{ }}
        $given = "10.sql = SELECT 'Hello World'\nform = {{form:S}}\n20.sql = SELECT 'Hello World2'\n30 {\nsql=SELECT 'Hello World'\n}\n   form=Person\n";
        $expected = "10.sql = SELECT 'Hello World'\nform = {{form:S}}\n20.sql = SELECT 'Hello World2'\n30.sql=SELECT 'Hello World'\nform=Person";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested: open bracket alone
        $given = "10.sql = SELECT 'Hello World'\n20\n{\nhead=test\n}\n30.sql = SELECT 'Hello World'\n";
        $expected = "10.sql = SELECT 'Hello World'\n20.head=test\n30.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Single open bracket inside a string.
        $given = "10.sql = SELECT 'Hello { World'";
        $expected = "10.sql = SELECT 'Hello { World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Complex test
        $given = "10.sql = SELECT '[\*]{7} [0-9]{5}<br>'\n20 {\n   10 {\n      5 {\n         sql = SELECT 'hello world<br>'\n      }\n   }\n}\n20.10.5.head = Terific\n20.sql = SELECT 20, '<br>'\n20.10.sql = SELECT  '20.10<br>'";
        $expected = "10.sql = SELECT '[\*]{7} [0-9]{5}<br>'\n20.10.5.sql = SELECT 'hello world<br>'\n20.10.5.head = Terific\n20.sql = SELECT 20, '<br>'\n20.10.sql = SELECT  '20.10<br>'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

    }

    public function testNestingToken() {
        $btp = new BodytextParser();

        // Nested expression: one level curly
        $given = "10 { \n     sql = SELECT 'Hello World' \n , 'next line', \n \n 'end' \n} \n";
        $expected = "10.sql = SELECT 'Hello World' , 'next line', 'end'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: one level angle
        $given = "#<\n10 < \n     sql = SELECT 'Hello World'\n>\n";
        $expected = "10.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: one level angle and single curly
        $given = "#<\n10 < \n  head = data { \n '1','2','3' \n }\n>\n";
        $expected = "10.head = data { '1','2','3' }";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: one level angle and single curly
        $given = "  #   < \n 10 < \n  sql = SELECT 'Hello World' \n>\n";
        $expected = "10.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: one level angle and single curly
        $given = "  #   > \n 10 < \n  sql = SELECT 'Hello World' \n>\n";
        $expected = "10.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: one level round bracket
        $given = "  #   ( \n 10 ( \n  sql = SELECT 'Hello World' \n)\n";
        $expected = "10.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: one level round bracket
        $given = "  #   ) \n 10 ( \n  sql = SELECT 'Hello World' \n)\n";
        $expected = "10.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: one level square bracket
        $given = "  #   [ \n 10 [ \n  sql = SELECT 'Hello World' \n]\n";
        $expected = "10.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: one level square bracket
        $given = "  #   ] \n 10 [ \n  sql = SELECT 'Hello World' \n]\n";
        $expected = "10.sql = SELECT 'Hello World'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: one level angle - garbage
        $given = "  #   < \n 10 { \n  sql = SELECT 'Hello World' \n}\n";
        $expected = "10 {\nsql = SELECT 'Hello World' }";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        // Nested expression: '</script>' is allowed here - with bad implemented parsing, it would detect a nesting token end, which is not the meant.
        $given = "  #   < \n 10 < \n  sql = SELECT 'Hello World' \n head = <script> \n>\n20.tail=</script>\n\n\n30.sql=SELECT 'something'";
        $expected = "10.sql = SELECT 'Hello World'\n10.head = <script>\n20.tail=</script>\n30.sql=SELECT 'something'";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        $open = '<';
        $close = '>';
        // muliple nesting, unnested rows  inbetween
        $given = <<<EOF
                # $open

                5.head = <h1>

                3.sql = SELECT ...

                10  $open
                  20 $open
                    sql = SELECT 10.20
                    head = <script>
                    tail = </script>
                    30 $open
                        head = <div>
                    $close
                  $close
                  30 $open
                    sql = SELECT 10.30
                  $close
                  sql = SELECT 10
                  50 $open
                    60 $open
                        tail = }
                        }
                        (:
                        }
                        ]
                        sql = SELECT 10.50.60
                        head = {
                          {
                          )
                          {
                          ;
                          [
                    $close
                    sql = SELECT 10.50
                    65 $open
                        sql = SELECT 10.50.65
                    $close
                  $close
                  40 $open
                    head = <table>
                  $close
                $close
                20.sql = SELECT 20
EOF;
        $expected = "5.head = <h1>\n3.sql = SELECT ...\n10.20.sql = SELECT 10.20\n10.20.head = <script>\n10.20.tail = </script>\n10.20.30.head = <div>\n10.30.sql = SELECT 10.30\n10.sql = SELECT 10\n10.50.60.tail = } } (: } ]\n10.50.60.sql = SELECT 10.50.60\n10.50.60.head = { { ) { ; [\n10.50.sql = SELECT 10.50\n10.50.65.sql = SELECT 10.50.65\n10.40.head = <table>\n20.sql = SELECT 20";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

    }

    public function testVariousNestingToken() {
        $btp = new BodytextParser();

        $tokenList = '{}[]<>()';
        for ($idx = 0; $idx < 4; $idx++) {
            $open = $tokenList[$idx * 2];
            $close = $tokenList[$idx * 2 + 1];

            // level open
            $given = <<<EOF
                # $open
                10 $open
                  sql = SELECT 'Hello World'
                $close
EOF;
            $expected = "10.sql = SELECT 'Hello World'";
            $result = $btp->process($given);
            $this->assertEquals($expected, $result);

            // level \n alone
            $given = <<<EOF
                # $open
                10
                $open
                  sql = SELECT 'Hello World'
                $close
EOF;
            $expected = "10.sql = SELECT 'Hello World'";
            $result = $btp->process($given);
            $this->assertEquals($expected, $result);

            // various linebreaks
            $given = <<<EOF
                # $open

                10  $open

                  sql = SELECT 'Hello World'

                $close

EOF;
            $expected = "10.sql = SELECT 'Hello World'";
            $result = $btp->process($given);
            $this->assertEquals($expected, $result);

            // multi line
            $given = <<<EOF
                # $open
                10  $open
                  sql = SELECT 'Hello World'
                        FROM Person

                        ORDER BY id

                        LIMIT 4
                   head = <div>
                $close
                10.tail = </div>
EOF;
            $expected = "10.sql = SELECT 'Hello World' FROM Person ORDER BY id LIMIT 4\n10.head = <div>\n10.tail = </div>";
            $result = $btp->process($given);
            $this->assertEquals($expected, $result);

            // mulitple nesting
            $given = <<<EOF
                # $open
                5.head = <h1>
                10  $open
                  sql = SELECT 'Hello World'
                  20 $open
                    sql = SELECT 'Hi'
                    head = <script>
                    tail = </script>
                    30 $open
                        head = <div>
                    $close

                  $close

                $close

EOF;
            $expected = "5.head = <h1>\n10.sql = SELECT 'Hello World'\n10.20.sql = SELECT 'Hi'\n10.20.head = <script>\n10.20.tail = </script>\n10.20.30.head = <div>";
            $result = $btp->process($given);
            $this->assertEquals($expected, $result);

            // muliple nesting, unnested rows  inbetween
            $given = <<<EOF
                # $open
                5.head = <h1>
                10  $open
                  sql = SELECT 'Hello World'
                  20 $open
                    sql = SELECT 'Hi'
                    head = <script>
                    tail = </script>
                    30 $open
                        head = <div>
                    $close
                  $close
                  30 $open
                    sql = SELECT 'After'
                  $close
                $close
                20.sql = SELECT ...

EOF;
            $expected = "5.head = <h1>\n10.sql = SELECT 'Hello World'\n10.20.sql = SELECT 'Hi'\n10.20.head = <script>\n10.20.tail = </script>\n10.20.30.head = <div>\n10.30.sql = SELECT 'After'\n20.sql = SELECT ...";
            $result = $btp->process($given);
            $this->assertEquals($expected, $result);

        }

    }

    public function testProcessPlainEscape() {
        $btp = new BodytextParser();

        // Simple row, nothing to remove
        $given = "10.sql = SELECT 'Hello World', 'p:id=1&\\\ngrId=2";
        $expected = "10.sql = SELECT 'Hello World', 'p:id=1&grId=2";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        $given = "10.sql = SELECT 'Hello World', 'p:id=1&  \\\n  grId=2  ";
        $expected = "10.sql = SELECT 'Hello World', 'p:id=1&grId=2";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);

        $given = "10.sql = SELECT 'Hello World', 'p:id=1&  \\\n  grId=2  \\\n 20.sql = SELECT ''";
        $expected = "10.sql = SELECT 'Hello World', 'p:id=1&grId=2\n20.sql = SELECT ''";
        $result = $btp->process($given);
        $this->assertEquals($expected, $result);
    }

    /**
     * @expectedException \qfq\UserFormException
     *
     */
    public function testProcessExceptionClose() {
        $btp = new BodytextParser();

        // Nested: unmatched close bracket
        $btp->process("10.sql = SELECT 'Hello World'\n } \n30.sql = SELECT 'Hello World'\n");

    }

    /**
     * @expectedException \qfq\UserFormException
     *
     */
    public function testProcessExceptionOpen() {
        $btp = new BodytextParser();

        // Nested: unmatched open bracket
        $btp->process("10.sql = SELECT 'Hello World'\n20 { \n30.sql = SELECT 'Hello World'\n");

    }

}
