<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 11/02/18
 * Time: 9:16 PM
 */

namespace qfq;

require_once(__DIR__ . '/../../qfq/helper/OnString.php');
require_once(__DIR__ . '/../../qfq/exceptions/UserFormException.php');

use qfq;

//use qfq\exceptions\CodeException;

//require_once(__DIR__ . '/../../qfq/store/Store.php');
//require_once(__DIR__ . '/../../qfq/Constants.php');
//require_once(__DIR__ . '/../../qfq/exceptions/CodeException.php');
//

use PHPUnit\Framework\TestCase;

//class OnStringTest extends \PHPUnit_Framework_TestCase {

/**
 * Class OnStringTest
 * @package qfq
 */
class OnStringTest extends TestCase {

    /**
     *
     */
    public function testStrrstr() {

        $this->assertEquals('', OnString::strrstr('', ''));
        $this->assertEquals('', OnString::strrstr('hello world to the limit', ''));
        $this->assertEquals('limit', OnString::strrstr('hello world to the limit', ' '));
        $this->assertEquals('', OnString::strrstr('', ' '));
    }

    /**
     *
     */
    public function testStripFirstCharIf() {
        $this->assertEquals('', OnString::stripFirstCharIf('', ''));
        $this->assertEquals('', OnString::stripFirstCharIf('c', ''));
        $this->assertEquals('', OnString::stripFirstCharIf('c', 'c'));
        $this->assertEquals('def', OnString::stripFirstCharIf('c', 'cdef'));
        $this->assertEquals('def', OnString::stripFirstCharIf('c', 'def'));
        $this->assertEquals('def', OnString::stripFirstCharIf('cd', 'def'));

    }

    /**
     *
     */
    public function testTrimQuote() {
        $this->assertEquals('', OnString::trimQuote(''));
        $this->assertEquals('test', OnString::trimQuote('test'));
        $this->assertEquals(' test ', OnString::trimQuote(' test '));
        $this->assertEquals('" test ', OnString::trimQuote('" test '));
        $this->assertEquals('" test \'', OnString::trimQuote('" test \''));
        $this->assertEquals('\' test "', OnString::trimQuote('\' test "'));
        $this->assertEquals(' test ', OnString::trimQuote('" test "'));
        $this->assertEquals(' test ', OnString::trimQuote("' test '"));
        $this->assertEquals(' te"st ', OnString::trimQuote("' te\"st '"));
        $this->assertEquals(' te\'st ', OnString::trimQuote("' te'st '"));

    }

    /**
     *
     */
    public function testRemoveNewlinesInNestedExpression() {
        $this->assertEquals("", OnString::removeNewlinesInNestedExpression(""));
        $this->assertEquals("test", OnString::removeNewlinesInNestedExpression("test"));
        $this->assertEquals("{{test}}", OnString::removeNewlinesInNestedExpression("{{test}}"));
        $this->assertEquals("{{test }}", OnString::removeNewlinesInNestedExpression("{{test\n}}"));
        $this->assertEquals("{{test two}}", OnString::removeNewlinesInNestedExpression("{{test\ntwo}}"));
        $this->assertEquals("Pre{{test two}}Post", OnString::removeNewlinesInNestedExpression("Pre{{test\ntwo}}Post"));
        $this->assertEquals("{{test{{two}}}}", OnString::removeNewlinesInNestedExpression("{{test{{two}}}}"));
        $this->assertEquals("{{a {{b }}}}",
            OnString::removeNewlinesInNestedExpression("{{a\n{{b\n}}}}"));
        $this->assertEquals("{{a b{{c d}}{{e f}} h}}",
            OnString::removeNewlinesInNestedExpression("{{a\nb{{c\nd}}{{e\nf}}\nh}}"));
        $this->assertEquals("param1=abc\nsqlInsert = {{SELECT * FROM test WHERE '{{var}}'='true'}}",
            OnString::removeNewlinesInNestedExpression("param1=abc\nsqlInsert = {{SELECT *\nFROM test\nWHERE '{{var}}'='true'}}"));
        $this->assertEquals("sqlInsert = {{SELECT * FROM test WHERE '{{var}}'='true'}}\nparam1=abc",
            OnString::removeNewlinesInNestedExpression("sqlInsert = {{SELECT *\nFROM test\nWHERE '{{var}}'='true'}}\nparam1=abc"));
    }

    /**
     * @expectedException \qfq\UserFormException
     *
     */
    public function testRemoveNewlinesInNestedExpression_missingClosing_1() {
        $str = OnString::removeNewlinesInNestedExpression("Hi! {{Test ");
    }
    /**
     * @expectedException \qfq\UserFormException
     *
     */
    public function testRemoveNewlinesInNestedExpression_missingClosing_2() {
        $str = OnString::removeNewlinesInNestedExpression("Hi! {{Test}");
    }
    /**
     * @expectedException \qfq\UserFormException
     *
     */
    public function testRemoveNewlinesInNestedExpression_missingClosing_3() {
        $str = OnString::removeNewlinesInNestedExpression("Hi! {{Test {{var }}");
    }

    /**
     * @expectedException \qfq\UserFormException
     *
     */
    public function testRemoveNewlinesInNestedExpression_missingOpening_1() {
        $str = OnString::removeNewlinesInNestedExpression("}}");
    }
    /**
     * @expectedException \qfq\UserFormException
     *
     */
    public function testRemoveNewlinesInNestedExpression_missingOpening_2() {
        $str = OnString::removeNewlinesInNestedExpression("{}}");
    }
    /**
     * @expectedException \qfq\UserFormException
     *
     */
    public function testRemoveNewlinesInNestedExpression_missingOpening_3() {
        $str = OnString::removeNewlinesInNestedExpression("{{Test}}}}");
    }
}